package edu.berkeley.sbp;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.Sequence.Position;
import edu.berkeley.sbp.GSS.Phase;
import edu.berkeley.sbp.GSS.Phase.Node;
import edu.berkeley.sbp.util.*;
import java.io.*;
import java.util.*;

/** thrown when the parser arrives at a state from which it is clear that no valid parse can result */
public class ParseFailed extends RuntimeException {
    private final Input.Location location;
    private final String message;
    public ParseFailed() { this("", null); }
    public ParseFailed(String message, Input.Location loc) { this.location = loc; this.message = message; }
    public Input.Location getLocation() { return location; }
    public String toString() { return message/* + (location==null ? "" : (" at " + location))*/; }

    public static <Tok> void complain(GSS.Phase<Tok>.Node n, HashMap<String,HashSet<String>> errors, boolean force) {
        //if (n.touched) return;
        //n.touched = true;
        for(Position p : n.state) {
            if (((p.isFirst() || p.isLast()) && !force) || p.owner().name==null) {
                for(Node n2 : n.parents())
                    complain(n2, errors, force | p.isFirst());
            } else {
                String seqname = p.owner().name;
                HashSet<String> hs = errors.get(seqname);
                if (hs==null) errors.put(seqname, hs = new HashSet<String>());
                hs.add(p.element()+"");
            }
        }
    }

    public static String el(Object e) {
        String s = e.toString();
        if (s.length()==0 || s.charAt(0)!='\"' || s.charAt(s.length()-1)!='\"') return ANSI.yellow(s);
        s = s.substring(1);
        s = s.substring(0, s.length()-1);
        StringBuffer ret = new StringBuffer();
        for(int i=0; i<s.length(); i++) {
            if (s.charAt(i)=='\\' && i<s.length()-1) ret.append(s.charAt(++i));
            else ret.append(s);
        }
        return ANSI.purple(ret.toString());
    }
    public static String error(String message, Object token, Iterable<Node> nodes) {
        String lookAhead = token==null ? "<EOF>" : token.toString();
        StringBuffer ret = new StringBuffer();
        ret.append("\n  ");
        ret.append(message);
        HashMap<String,HashSet<String>> errors = new HashMap<String,HashSet<String>>();
        for(Node n : nodes) {
            //System.out.println(n.state);
            complain(n, errors, false);
        }
        for(String s : errors.keySet()) {
            ret.append("    while parsing " + ANSI.yellow(s));
            HashSet<String> hs = errors.get(s);
            if (hs.size()==1) ret.append(" expected " + ANSI.yellow(el(hs.iterator().next())) + "\n");
            else {
                ret.append(" expected ");
                boolean first = true;
                for(String s2 : hs) {
                    if (!first) ret.append(" or ");
                    first = false;
                    ret.append(ANSI.yellow(el(s2)));
                }
                ret.append("\n");
            }
        }
        return ret.toString();
    }

}
