package edu.berkeley.sbp.chr;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.lang.ref.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.misc.*;
import edu.berkeley.sbp.Input.Location;

public class CharRange extends Atom<Character> {

    public CharRange(char a) { this(a,a); }
    public CharRange(char a, char b) { this(new CharTopology(a, b)); }
    public CharRange(Topology<Character> t) { this.t = t; }

    private Topology<Character> t;
    public  Topology<Character> top() { return t; }

    public static final char left       = (char)9998;
    public static final char right      = (char)9999;
    public static final Atom leftBrace  = new CharRange(left,left)   { public String toString() { return "{"; } };
    public static final Atom rightBrace = new CharRange(right,right) { public String toString() { return "}"; } };

    public static Atom set(Range.Set r) { return new CharRange(new CharTopology(r)); }

    public String toString() { return t.toString(); }

    /** returns an element which exactly matches the string given */
    public static Element string(String s) {
        if (s.length() == 0) return Union.epsilon;
        final String escapified = "\""+StringUtil.escapify(s, "\"\r\n\\")+"\"";
        Element ret;
        if (s.length() == 1) {
            ret =
                new CharRange(s.charAt(0)) {
                    public String toString() { return escapified; } };
        } else {
            Union ret2 = new Union("\""+s+"\"_str", true) {
                    public String toString() { return escapified; } };
            Element[] refs = new Element[s.length()];
            for(int i=0; i<refs.length; i++) refs[i] = new CharRange(s.charAt(i));
            ret2.add(Sequence.constant(refs, s, null, null));
            ret = ret2;
        }
        return ret;
    }

}
