package edu.berkeley.sbp.misc;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.lang.ref.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.Input.Location;
import edu.berkeley.sbp.util.*;

public abstract class CartesianInput<Tok> implements Input<Tok> {

    private int line  = 1;
    private int col   = 0;

    public abstract Tok     next() throws IOException;
    public abstract boolean isCR();

    long then = 0;
    private Input.Location location = new LocWrap(line, col);
    public Input.Location getLocation() { return location; }
    public Tok next(int numstates, int resets, int waits) throws IOException {
        Tok t = next();
        if (t==null) return null;
        String s = "  line "+line+", col " + col;
        while(s.length() < 20) s += " ";
        s += "[ambiguity level: " + (numstates-1) + "] [resets: " + resets + "] [waits: " + waits + "]";
        long now = System.currentTimeMillis();
        if (now-then > 10) {
            then = now;
            System.out.print(s + "                                \r");
        }
        if (isCR()) { 
            line++;
            col = 1;
        } else {
            col++;
        }
        location = new LocWrap(line, col);
        return t;
    }

    private class LocWrap implements Input.Location {
        public final int line;
        public final int col;
        public String toString()            { return line + ":" + col; }
        public LocWrap(int line, int col) { this.line = line; this.col = col; }
    }
}
